#!/usr/bin/env python
#======================================================================
# FILE: Duration.py
# CREATOR: eric
#
# (C) COPYRIGHT 2001, Eric Busboom <eric@civicknowledge.com>
# (C) COPYRIGHT 2001, Patrick Lewis <plewis@inetarena.com>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#===============================================================

from LibicalWrap import *
from Property import Property
from types import DictType, StringType, IntType

class Duration(Property):
    """
    Represent a length of time, like 3 minutes, or 6 days, 20 seconds.


    """

    def __init__(self, arg, name="DURATION"):
        """
        Create a new duration from an RFC2445 string or number of seconds.
        Construct the duration from an iCalendar string or a number of seconds.

        Duration("P3DT2H34M45S")   Construct from an iCalendar string
        Duration(3660)             Construct from seconds
        """

        self.dur = None

        e=icalerror_supress("MALFORMEDDATA")

        if isinstance(arg, DictType):

            self.dur = icaldurationtype_from_string(arg['value'])
            Property.__init__(self,ref=arg['ref'])
        else:
            if isinstance(arg, StringType):
                self.dur = icaldurationtype_from_string(arg)
            elif isinstance(arg, IntType):
                self.dur = icaldurationtype_from_int(arg)
            elif isinstance(arg,Duration):
                self.dur = arg.dur
            else:
                self.dur = icaldurationtype_null_duration()

            Property.__init__(self,type=name)

        icalerror_restore("MALFORMEDDATA",e)

        if self.dur == None or icaldurationtype_is_null_duration(self.dur):
            raise Property.ConstructorFailedError("Failed to construct Duration from " +str(arg))

        try:
            self._update_value()
        except Property.UpdateFailedError:
            raise Property.ConstructorFailedError("Failed to construct Duration from  " + str(arg))

    def _update_value(self):

        self.value(icaldurationtype_as_ical_string(self.dur),"DURATION")

    def valid(self):
        "Returns true if this is a valid duration"

        return not icaldurationtype_is_null_duration(self.dur)

    def seconds(self,v=None):
        """Return or set duration in seconds"""
        if(v != None):
            self.dur = icaldurationtype_from_int(v);
            self.dict['value'] = icaltimedurationtype_as_ical_string(self.dur)
        return icaldurationtype_as_int(self.dur)



#!/usr/bin/env python
#======================================================================
# FILE: Libical.py
# CREATOR: eric
#
# (C) COPYRIGHT 2001, Eric Busboom <eric@civicknowledge.com>
# (C) COPYRIGHT 2001, Patrick Lewis <plewis@inetarena.com>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#======================================================================

from LibicalWrap import ICAL_PACKAGE, ICAL_VERSION
from Component import Component, NewComponent, Event, Todo, Journal
from Property import Property, RecurrenceSet, test_enum
from Time import Time, UTC
from Period import Period
from Duration import Duration
from Attendee import Attendee, Organizer
from DerivedProperties import RDate, Trigger,Recurrence_Id, Attach
from Store import Store, FileStore
from Gauge import Gauge

version = ICAL_VERSION



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./error.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

message = "This is a stinky error!";

string_rep = ICalGLib.error_strerror(ICalGLib.ErrorEnum.NEWFAILED_ERROR);
assert(string_rep == "NEWFAILED: Failed to create a new object via a *_new() routine")
string_perror = ICalGLib.error_perror();
assert(string_perror == "NO: No error");

ICalGLib.error_set_error_state(ICalGLib.ErrorEnum.NEWFAILED_ERROR, ICalGLib.ErrorState.FATAL);
state = ICalGLib.error_get_error_state(ICalGLib.ErrorEnum.NEWFAILED_ERROR);
assert(state == ICalGLib.ErrorState.FATAL);

ICalGLib.error_stop_here();
enumeration = ICalGLib.errno_return();
string_rep = ICalGLib.error_strerror(enumeration);
assert(string_rep == "NO: No error");

state = ICalGLib.error_supress(message);
ICalGLib.error_restore(message, state);



#!/usr/bin/env python
#======================================================================
# FILE: Collection.py
# CREATOR: eric
#
# (C) COPYRIGHT 2001, Eric Busboom <eric@civicknowledge.com>
# (C) COPYRIGHT 2001, Patrick Lewis <plewis@inetarena.com>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#======================================================================

from types import *

class Collection:
    """A group of components that can be modified somewhat like a list.

    Usage:
        Collection(componet, propSequence)

    component is a Component object
    propSequence is a list or tuple of Property (or subclass of Property)
        of objects already in component
    """

    def __init__(self, component, propSequence):
        self._properties = list(propSequence[:])
        self._component = component

    def __getslice__(self, beg, end):
        return Collection(self._component, self._properties[beg:end])

    def __setslice__(self, beg, end, sequence):

        if  not isinstance(sequence,ListType):
            raise TypeError, "must assign list (not instance) to slice"

        oldProps = self._properties[beg:end]

        for p in oldProps:
            self._component.remove_property(p)

        self._properties[beg:end] = sequence
        for p in sequence:
            self._component.add_property(p)

    def __getitem__(self, i):
        return self._properties[i]

    def __setitem__(self, i, prop):
        self._component.remove_property(self._properties[i])
        self._component.add_property(prop)
        self._properties[i]=prop

    def __delitem__(self, i):
        self._component.remove_property(self._properties[i])
        del self._properties[i]

    def __len__(self):
        return len(self._properties)

    def __str__(self):
        s = "[ "
        if self._properties:
            s = s + str(self._properties[0])
            for p in self._properties[1:]:
                s = "%s, %s" % (s, p)
        s = s + " ]"
        return s

    def append(self, property):
        self._properties.append(property)
        self._component.add_property(property)

class ComponentCollection:

    def __init__(self, parent, componentSequence):
        self._parent = parent
        self._components = list(componentSequence[:])

    def __getslice__(self, beg, end):
        return ComponentCollection(self._parent, self._components[beg:end])

    def __setslice__(self, beg, end, sequence):
        oldComps = self._components[beg:end]
        self._components.__setslice__(beg, end, sequence)
        for c in sequence:
            self._parent.add_component(c)
        for c in oldComps:
            self._parent.remove_component(c)

    def __getitem__(self, i):
        return self._components[i]

    def __setitem__(self, i, prop):
        self._parent.remove_component(self._components[i])
        self._parent.add_property(prop)
        self._components[i]=prop

    def __delitem__(self, i):
        self._parent.remove_component(self._components[i])
        del self._components[i]

    def __len__(self):
        return len(self._components)

    def __add__(self, iterable):
        for i in iterable:
            self.append(i)

    def append(self, property):
        self._components.append(property)
        self._parent.add_component(property)



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./value.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

kind = ICalGLib.ValueKind.ATTACH_VALUE;
value = ICalGLib.Value.new(kind);
assert(value.is_valid() == 1);
assert(value.isa() == kind);
assert(value.isa_value() == 1);

string = "This is a link";
value_from_string = ICalGLib.Value.new_from_string(kind, string);
assert(value_from_string.as_ical_string() == string);
assert(value_from_string.is_valid() == 1);
assert(value_from_string.isa() == kind);
assert(value_from_string.isa_value() == 1);

value_from_string_clone = value_from_string.clone();
assert(value_from_string_clone.as_ical_string() == string);
assert(value_from_string_clone.is_valid() == 1);
assert(value_from_string_clone.isa() == kind);
assert(value_from_string_clone.isa_value() == 1);

value_a = ICalGLib.Value.new(ICalGLib.ValueKind.RECUR_VALUE);
value_b = ICalGLib.Value.new(ICalGLib.ValueKind.ATTACH_VALUE);
compare_result = ICalGLib.Value.compare(value_a, value_b);
assert(compare_result == ICalGLib.ParameterXliccomparetype.NOTEQUAL);

string_a = "a string";
string_b = "a string";
value_a = ICalGLib.Value.new_from_string(kind, string_a);
value_b = ICalGLib.Value.new_from_string(kind, string_b);
compare_result = ICalGLib.Value.compare(value_a, value_b);
assert(compare_result == ICalGLib.ParameterXliccomparetype.EQUAL);

string_a = "a string";
string_b = "b string";
value_a = ICalGLib.Value.new_from_string(kind, string_a);
value_b = ICalGLib.Value.new_from_string(kind, string_b);
compare_result = ICalGLib.Value.compare(value_a, value_b);
assert(compare_result == ICalGLib.ParameterXliccomparetype.NOTEQUAL);

kind_string = ICalGLib.Value.kind_to_string(kind);
assert(kind_string == "ATTACH");
converted_kind = ICalGLib.Value.kind_from_string(kind_string);
assert(converted_kind == kind);

szText = "This is a text\nand this is a new line";
after_encoded_szText = "This is a text\\nand this is a new line";
before_decoded_szText = "This is a text\\\nand this is a new line";
result = ICalGLib.Value.encode_ical_string(szText);
assert(result == after_encoded_szText);

result = ICalGLib.Value.decode_ical_string(before_decoded_szText);
assert(result == szText);

szText = "Simple text";
result = ICalGLib.Value.encode_ical_string(szText);
assert result == szText
result = ICalGLib.Value.decode_ical_string(result);
assert result == szText



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./timezone.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

import os
import sys

try:
    zoneinfodir = os.environ['ZONEINFO_DIRECTORY']
except KeyError:
    print("Error: The ZONEINFO_DIRECTORY environment variable isn't set")
    sys.exit(1)
if not os.path.isdir(zoneinfodir):
    print("Error: The ZONEINFO_DIRECTORY environment variable isn't properly set")
    sys.exit(1)
ICalGLib.Timezone.set_zone_directory(zoneinfodir);
ICalGLib.Timezone.set_tzid_prefix("/citadel.org/");

la = ICalGLib.Timezone.get_builtin_timezone("America/Los_Angeles");
chicago = ICalGLib.Timezone.get_builtin_timezone("America/Chicago");
assert la.get_tzid().find("Los_Angeles") != -1;
assert la.get_location() == "America/Los_Angeles";
assert la.get_tznames() == "PST/PDT";

la_copy = la.copy();
assert la_copy.get_tzid() == la.get_tzid();
assert la_copy.get_location() == la.get_location();

timezones = ICalGLib.Timezone.get_builtin_timezones();
timezone = ICalGLib.Timezone.array_element_at(timezones, 0);
if not ICalGLib.Timezone.get_builtin_tzdata():
    assert timezone.get_display_name() == "Europe/Andorra";
assert timezones.size() > 0;

from_tzid = ICalGLib.Timezone.get_builtin_timezone_from_tzid(la.get_tzid());
assert from_tzid.get_location() == "America/Los_Angeles";

utc = ICalGLib.Timezone.get_utc_timezone();
assert utc.get_display_name() == "UTC";
utc2 = ICalGLib.Timezone.get_utc_timezone();
assert utc == utc2

time = ICalGLib.Time.new();
before = time.get_hour();
ICalGLib.Time.convert_timezone(time, la, chicago);
after = time.get_hour();
assert abs(after - before) == 2;
ICalGLib.Time.set_timezone(time, utc);
assert ICalGLib.Time.get_timezone(time) == utc;
ICalGLib.Time.set_timezone(time, la);
assert ICalGLib.Time.get_timezone(time) == la;

timeclone = time.clone()
assert time != timeclone
time = ICalGLib.Time.convert_to_zone(time, chicago)
timeclone.convert_to_zone_inplace(chicago)
assert time.get_year() == timeclone.get_year();
assert time.get_month() == timeclone.get_month();
assert time.get_day() == timeclone.get_day();
assert time.get_hour() == timeclone.get_hour();
assert time.get_minute() == timeclone.get_minute();
assert time.get_second() == timeclone.get_second();
assert time.get_timezone() == timeclone.get_timezone();
assert time.is_date() == timeclone.is_date();
assert time.is_daylight() == timeclone.is_daylight();
assert time.is_utc() == timeclone.is_utc();

timeclone = ICalGLib.Time.clone(time);
assert time != timeclone;
assert ICalGLib.Time.get_year(time) == ICalGLib.Time.get_year(timeclone);
assert ICalGLib.Time.get_month(time) == ICalGLib.Time.get_month(timeclone);
assert ICalGLib.Time.get_day(time) == ICalGLib.Time.get_day(timeclone);
assert ICalGLib.Time.get_hour(time) == ICalGLib.Time.get_hour(timeclone);
assert ICalGLib.Time.get_minute(time) == ICalGLib.Time.get_minute(timeclone);
assert ICalGLib.Time.get_second(time) == ICalGLib.Time.get_second(timeclone);
assert ICalGLib.Time.get_timezone(time) == ICalGLib.Time.get_timezone(timeclone);
assert ICalGLib.Time.is_date(time) == ICalGLib.Time.is_date(timeclone);
assert ICalGLib.Time.is_daylight(time) == ICalGLib.Time.is_daylight(timeclone);
assert ICalGLib.Time.is_utc(time) == ICalGLib.Time.is_utc(timeclone);

time.set_date(2019, 1, 24)
assert time.get_year() == 2019
assert time.get_month() == 1
assert time.get_day() == 24

array = time.get_date()
assert len(array) == 3
assert array[0] == 2019
assert array[1] == 1
assert array[2] == 24

time.set_time(12, 11, 10)
assert time.get_hour() == 12
assert time.get_minute() == 11
assert time.get_second() == 10

array = time.get_time()
assert len(array) == 3
assert array[0] == 12
assert array[1] == 11
assert array[2] == 10

component = la.get_component();
timezone = ICalGLib.Timezone.new();
timezone.set_component(ICalGLib.Component.clone(component));
assert timezone.get_location() == la.get_location();

array = ICalGLib.Timezone.array_new();
ICalGLib.Timezone.array_append_from_vtimezone(array, ICalGLib.Component.clone(la.get_component()));
ICalGLib.Timezone.array_append_from_vtimezone(array, ICalGLib.Component.clone(chicago.get_component()));
assert array.size() == 2;
timezone1 = ICalGLib.Timezone.array_element_at(array, 0);
assert timezone1.get_display_name() == la.get_display_name();
timezone2 = ICalGLib.Timezone.array_element_at(array, 1);
assert timezone2.get_display_name() == chicago.get_display_name();

ICalGLib.Timezone.free_builtin_timezones()
ICalGLib.Object.free_global_objects()



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./misc.py

###############################################################################
#
# Copyright (C) 2019 Red Hat Inc. <www.redhat.com>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

geo = ICalGLib.Geo.new(10.0, 20.0)
assert geo.get_lat() == 10.0
assert geo.get_lon() == 20.0
geo_clone = geo.clone()
assert geo.get_lat() == geo_clone.get_lat()
assert geo.get_lon() == geo_clone.get_lon()
geo_clone.set_lat(30.0)
geo_clone.set_lon(40.0);
assert geo_clone.get_lat() == 30.0
assert geo_clone.get_lon() == 40.0
assert geo.get_lat() != geo_clone.get_lat()
assert geo.get_lon() != geo_clone.get_lon()

start = ICalGLib.Time.new_from_string("20190130T111213Z");
end = ICalGLib.Time.new_from_string("20190203T100908Z")
span = ICalGLib.TimeSpan.new(start, end, 0);
assert span.get_start() == start.as_timet()
assert span.get_end() == end.as_timet()
assert span.get_is_busy() == 0
start = ICalGLib.Time.new_from_string("20190330T131415Z");
end = ICalGLib.Time.new_from_string("20190403T070605Z")
span = ICalGLib.TimeSpan.new(start, end, 1);
assert span.get_start() == start.as_timet()
assert span.get_end() == end.as_timet()
assert span.get_is_busy() == 1

span = ICalGLib.TimeSpan.new_timet(11, 22, 1)
assert span.get_start() == 11
assert span.get_end() == 22
assert span.get_is_busy() == 1
span_clone = span.clone()
assert span.get_start() == span_clone.get_start()
assert span.get_end() == span_clone.get_end()
assert span.get_is_busy() == span_clone.get_is_busy()
span_clone.set_start(33)
span_clone.set_end(44)
span_clone.set_is_busy(0)
assert span_clone.get_start() == 33
assert span_clone.get_end() == 44
assert span_clone.get_is_busy() == 0
assert span.get_start() != span_clone.get_start()
assert span.get_end() != span_clone.get_end()
assert span.get_is_busy() != span_clone.get_is_busy()
span = ICalGLib.TimeSpan.new_timet(55, 66, 0)
assert span.get_start() == 55
assert span.get_end() == 66
assert span.get_is_busy() == 0



#!/usr/bin/env python
#======================================================================
# FILE: Gauge.py
# CREATOR: mtearle
#
# (C) COPYRIGHT 2001, Eric Busboom <eric@civicknowledge.com>
# (C) COPYRIGHT 2001, Patrick Lewis <plewis@inetarena.com>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#======================================================================

from LibicalWrap import *
from Error import LibicalError
from Component import Component

class Gauge:
    """
    Base class for gauge
    """

    class ConstructorFailedError(LibicalError):
        "Failed to create a Guage "

    class CloneFailedError(LibicalError):
        "Failed to clone a component given Gauge "

    class CompareFailedError(LibicalError):
        "Failed to compare a component given Gauge "

    def __init__(self,ref=None,sql=None,expand=0):
        if ref != None:
            self._ref = ref
        elif sql != None:
            s = str(sql)
            self._ref = icalgauge_new_from_sql(s,expand)
        else:
            Gauge.ConstructorFailedError("No SQL Specified")

    def __del__(self):
        if self._ref != None:
            icalgauge_free(self._ref)
            self._ref = None

    def ref(self):
        return self._ref

    def compare(self, comp):
        if not isinstance(comp,Component):
            raise Gauge.CompareFailedError("Argument is not a component")

        if comp.ref() == None:
            raise Gauge.CompareFailedError("Argument is not a component")

        return icalgauge_compare(self._ref, comp.ref())



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./comprehensive.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

event_str1 = \
    "BEGIN:VEVENT\n"                                \
    "UID:event-uid-123\n"                           \
    "SUMMARY;LANGUAGE=en-US:parent\n"               \
    "DTSTART;TZID=Europe/Prague:20140306T090000\n"  \
    "DTEND;TZID=Europe/Berlin:20140306T093000\n"    \
    "CLASS:PUBLIC\n"                                \
    "PRIORITY:5\n"                                  \
    "DTSTAMP:20140207T020756Z\n"                    \
    "TRANSP:OPAQUE\n"                               \
    "STATUS:CONFIRMED\n"                            \
    "SEQUENCE:0\n"                                  \
    "LOCATION;LANGUAGE=en-US:Location\n"            \
    "END:VEVENT\n"

event_str2 = \
    "BEGIN:VEVENT\n"                                \
    "UID:event-uid-123\n"                           \
    "SUMMARY;LANGUAGE=en-US:childEvent1\n"          \
    "DTSTART;TZID=Europe/Prague:20140306T090000\n"  \
    "DTEND;TZID=Europe/Berlin:20140306T093000\n"    \
    "CLASS:PUBLIC\n"                                \
    "PRIORITY:5\n"                                  \
    "DTSTAMP:20140207T020756Z\n"                    \
    "TRANSP:OPAQUE\n"                               \
    "STATUS:CONFIRMED\n"                            \
    "SEQUENCE:0\n"                                  \
    "LOCATION;LANGUAGE=en-US:Location\n"            \
    "END:VEVENT\n"

event_str3 = \
    "BEGIN:VEVENT\n"                                \
    "UID:event-uid-123\n"                           \
    "SUMMARY;LANGUAGE=en-US:childEvent2\n"          \
    "DTSTART;TZID=Europe/Prague:20140306T090000\n"  \
    "DTEND;TZID=Europe/Berlin:20140306T093000\n"    \
    "CLASS:PUBLIC\n"                                \
    "PRIORITY:5\n"                                  \
    "DTSTAMP:20140207T020756Z\n"                    \
    "TRANSP:OPAQUE\n"                               \
    "STATUS:CONFIRMED\n"                            \
    "SEQUENCE:0\n"                                  \
    "LOCATION;LANGUAGE=en-US:Location\n"            \
    "END:VEVENT\n"

event_str4 = \
    "BEGIN:VEVENT\n"                                \
    "UID:event-uid-123\n"                           \
    "SUMMARY;LANGUAGE=en-US:childEvent3\n"          \
    "DTSTART;TZID=Europe/Prague:20140306T090000\n"  \
    "DTEND;TZID=Europe/Berlin:20140306T093000\n"    \
    "CLASS:PUBLIC\n"                                \
    "PRIORITY:5\n"                                  \
    "DTSTAMP:20140207T020756Z\n"                    \
    "TRANSP:OPAQUE\n"                               \
    "STATUS:CONFIRMED\n"                            \
    "SEQUENCE:0\n"                                  \
    "LOCATION;LANGUAGE=en-US:Location\n"            \
    "END:VCALENDAR\n"

event_str5 = \
    "BEGIN:VCALENDAR\n"                             \
    "UID:event-uid-123\n"                           \
    "SUMMARY;LANGUAGE=en-US:childCalendar1\n"       \
    "DTSTART;TZID=Europe/Prague:20140306T090000\n"  \
    "DTEND;TZID=Europe/Berlin:20140306T093000\n"    \
    "CLASS:PUBLIC\n"                                \
    "PRIORITY:5\n"                                  \
    "DTSTAMP:20140207T020756Z\n"                    \
    "TRANSP:OPAQUE\n"                               \
    "STATUS:CONFIRMED\n"                            \
    "SEQUENCE:0\n"                                  \
    "LOCATION;LANGUAGE=en-US:Location\n"            \
    "END:VCALENDAR\n"

combined_string = \
    "BEGIN:VEVENT\r\n"                              \
    "UID:event-uid-123\r\n"                         \
    "SUMMARY;LANGUAGE=en-US:parent\r\n"             \
    "DTSTART;TZID=Europe/Prague:20140306T090000\r\n"\
    "DTEND;TZID=Europe/Berlin:20140306T093000\r\n"  \
    "CLASS:PUBLIC\r\n"                              \
    "PRIORITY:5\r\n"                                \
    "DTSTAMP:20140207T020756Z\r\n"                  \
    "TRANSP:OPAQUE\r\n"                             \
    "STATUS:CONFIRMED\r\n"                          \
    "SEQUENCE:0\r\n"                                \
    "LOCATION;LANGUAGE=en-US:Location\r\n"          \
    "BEGIN:VEVENT\r\n"                              \
    "UID:event-uid-123\r\n"                         \
    "SUMMARY;LANGUAGE=en-US:childEvent1\r\n"        \
    "DTSTART;TZID=Europe/Prague:20140306T090000\r\n" \
    "DTEND;TZID=Europe/Berlin:20140306T093000\r\n"  \
    "CLASS:PUBLIC\r\n"                              \
    "PRIORITY:5\r\n"                                \
    "DTSTAMP:20140207T020756Z\r\n"                  \
    "TRANSP:OPAQUE\r\n"                             \
    "STATUS:CONFIRMED\r\n"                          \
    "SEQUENCE:0\r\n"                                \
    "LOCATION;LANGUAGE=en-US:Location\r\n"          \
    "END:VEVENT\r\n"                                \
    "BEGIN:VEVENT\r\n"                              \
    "UID:event-uid-123\r\n"                         \
    "SUMMARY;LANGUAGE=en-US:childEvent2\r\n"        \
    "DTSTART;TZID=Europe/Prague:20140306T090000\r\n" \
    "DTEND;TZID=Europe/Berlin:20140306T093000\r\n"  \
    "CLASS:PUBLIC\r\n"                              \
    "PRIORITY:5\r\n"                                \
    "DTSTAMP:20140207T020756Z\r\n"                  \
    "TRANSP:OPAQUE\r\n"                             \
    "STATUS:CONFIRMED\r\n"                          \
    "SEQUENCE:0\r\n"                                \
    "LOCATION;LANGUAGE=en-US:Location\r\n"          \
    "END:VEVENT\r\n"                                \
    "BEGIN:VEVENT\r\n"                              \
    "UID:event-uid-123\r\n"                         \
    "SUMMARY;LANGUAGE=en-US:childEvent3\r\n"        \
    "DTSTART;TZID=Europe/Prague:20140306T090000\r\n" \
    "DTEND;TZID=Europe/Berlin:20140306T093000\r\n"  \
    "CLASS:PUBLIC\r\n"                              \
    "PRIORITY:5\r\n"                                \
    "DTSTAMP:20140207T020756Z\r\n"                  \
    "TRANSP:OPAQUE\r\n"                             \
    "STATUS:CONFIRMED\r\n"                          \
    "SEQUENCE:0\r\n"                                \
    "LOCATION;LANGUAGE=en-US:Location\r\n"          \
    "END:VEVENT\r\n"                                \
    "BEGIN:VCALENDAR\r\n"                           \
    "UID:event-uid-123\r\n"                         \
    "SUMMARY;LANGUAGE=en-US:childCalendar1\r\n"     \
    "DTSTART;TZID=Europe/Prague:20140306T090000\r\n" \
    "DTEND;TZID=Europe/Berlin:20140306T093000\r\n"  \
    "CLASS:PUBLIC\r\n"                              \
    "PRIORITY:5\r\n"                                \
    "DTSTAMP:20140207T020756Z\r\n"                  \
    "TRANSP:OPAQUE\r\n"                             \
    "STATUS:CONFIRMED\r\n"                          \
    "SEQUENCE:0\r\n"                                \
    "LOCATION;LANGUAGE=en-US:Location\r\n"          \
    "END:VCALENDAR\r\n"                             \
    "END:VEVENT\r\n"

def main():
    #Test child component manipulation
    parent = ICalGLib.Component.new_from_string(event_str1);
    comp1 = ICalGLib.Component.new_from_string(event_str2);
    comp2 = ICalGLib.Component.new_from_string(event_str3);
    comp3 = ICalGLib.Component.new_from_string(event_str4);
    comp4 = ICalGLib.Component.new_from_string(event_str5);

    parent.add_component(comp1);
    parent.add_component(comp2);
    parent.add_component(comp3);
    parent.add_component(comp4);

    assert parent.as_ical_string() == combined_string;

    count = parent.count_components(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    child_component = parent.get_first_component(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    for i in range(0, count):
        if (child_component.get_summary() == "childEvent2"):
            child_component.set_summary("childEventTwo");

            start = ICalGLib.Time.new_from_string("20141115T211923");
            end = ICalGLib.Time.new_from_string("20141115T221923");
            child_component.set_dtstart(start);
            child_component.set_dtend(end);

            child_component.set_dtstamp(start);
            child_component.set_location("East Lansing, MI, US");

            child_component.set_relcalid("relcalid for childEventTwo");
            recur_string = "RRULE:FREQ=DAILY;INTERVAL=10;COUNT=5";
            property = ICalGLib.Property.new_from_string(recur_string);
            child_component.add_property(property);
        if (i != count-1):
            child_component = parent.get_next_component(ICalGLib.ComponentKind.VEVENT_COMPONENT);

    modifiedCombinedString = parent.as_ical_string();
    newParent = ICalGLib.Component.new_from_string(modifiedCombinedString);

    count = parent.count_components(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    child_component = parent.get_first_component(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    for i in range(0, count):
        if (child_component.get_summary() == "childEventTwo"):
            child_component.set_summary("childEventTwo");

            dtstart = child_component.get_dtstart();
            start_string = ICalGLib.Time.as_ical_string(dtstart);
            assert(start_string == "20141115T211923");
            dtend = child_component.get_dtend();
            end_string = ICalGLib.Time.as_ical_string(dtend);
            assert(end_string == "20141115T221923");

            timestamp = child_component.get_dtstamp();
            assert(ICalGLib.Time.as_ical_string(timestamp) == "20141115T211923");
            assert(child_component.get_location() == "East Lansing, MI, US");
            assert(child_component.get_relcalid() == "relcalid for childEventTwo");

            recurProperty = child_component.get_first_property(ICalGLib.PropertyKind.RRULE_PROPERTY);
            assert recurProperty.as_ical_string() == "RRULE:FREQ=DAILY;COUNT=5;INTERVAL=10\r\n";
        if (i != count-1):
            child_component = parent.get_next_component(ICalGLib.ComponentKind.VEVENT_COMPONENT);

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./period-type.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

string = "19970101T183248Z/19970102T071625Z";

period = ICalGLib.Period.new_from_string(string);
retrieved_string = period.as_ical_string();
assert(retrieved_string == string);

null_period = ICalGLib.Period.new_null_period();
assert(null_period.is_null_period() == 1);
assert(period.is_null_period() == 0);

assert(null_period.is_valid_period() == 1);
assert(period.is_valid_period() == 1);

start = period.get_start();
assert(start.get_year() == 1997);
assert(start.get_month() == 1);
assert(start.get_day() == 1);
assert(start.get_hour() == 18);
assert(start.get_minute() == 32);
assert(start.get_second() == 48);

end = period.get_end();
assert(end.get_year() == 1997);
assert(end.get_month() == 1);
assert(end.get_day() == 2);
assert(end.get_hour() == 7);
assert(end.get_minute() == 16);
assert(end.get_second() == 25);

duration = period.get_duration();
assert(duration.as_int() == 0);

string = "19970101T182346Z/PT5H30M";
period = ICalGLib.Period.new_from_string(string);
retrieved_string = period.as_ical_string();
assert(retrieved_string == string);

start = period.get_start();
assert(start.get_year() == 1997);
assert(start.get_month() == 1);
assert(start.get_day() == 1);
assert(start.get_hour() == 18);
assert(start.get_minute() == 23);
assert(start.get_second() == 46);

end = period.get_end();
assert(end.get_year() == 0);
assert(end.get_month() == 0);
assert(end.get_day() == 0);
assert(end.get_hour() == 0);
assert(end.get_minute() == 0);
assert(end.get_second() == 0);

duration = period.get_duration();
assert(duration.as_ical_string() == "PT5H30M");



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./property.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

action_property = ICalGLib.Property.new(ICalGLib.PropertyKind.ACKNOWLEDGED_PROPERTY);
action_property_clone = action_property.clone();
assert(action_property_clone.as_ical_string() == action_property.as_ical_string());

property_string = "SUMMARY:Bastille Day Party";
string_property = ICalGLib.Property.new_from_string(property_string);
string_property_clone = string_property.clone();
assert(string_property.as_ical_string() == string_property_clone.as_ical_string());
assert(string_property.isa() == ICalGLib.PropertyKind.SUMMARY_PROPERTY);
assert(string_property.isa_property() == 1);
assert(string_property.get_property_name() == "SUMMARY");

altrep1 = "This is an altrep 1";
parameter1 = ICalGLib.Parameter.new_altrep(altrep1);
altrep2 = "This is an altrep 2";
parameter2 = ICalGLib.Parameter.new_altrep(altrep2);
string_property.add_parameter(parameter1);
string_property.add_parameter(parameter2);
string_property.set_parameter_from_string("ACTIONPARAM", "This is a action param");

assert(string_property.count_parameters() == 3);
retrieved_parameter1 = string_property.get_first_parameter(ICalGLib.ParameterKind.ALTREP_PARAMETER);
assert(retrieved_parameter1.as_ical_string() == parameter1.as_ical_string());
retrieved_parameter2 = string_property.get_next_parameter(ICalGLib.ParameterKind.ALTREP_PARAMETER);
assert(retrieved_parameter2.as_ical_string() == parameter2.as_ical_string());
retrieved_parameter3 = string_property.get_first_parameter(ICalGLib.ParameterKind.ACTIONPARAM_PARAMETER);
assert(retrieved_parameter3.as_ical_string() == "ACTIONPARAM=This is a action param");

string_property.remove_parameter_by_kind(ICalGLib.ParameterKind.CHARSET_PARAMETER);
assert(string_property.count_parameters() == 3);
string_property.remove_parameter_by_kind(ICalGLib.ParameterKind.ALTREP_PARAMETER);
assert(string_property.count_parameters() == 2);
string_property.remove_parameter_by_name("ACTIONPARAM");
assert(string_property.count_parameters() == 1);

kind = ICalGLib.ValueKind.ATTACH_VALUE;
string = "This is a link";
value_from_string = ICalGLib.Value.new_from_string(kind, string);
string_property.set_value(value_from_string);
string_from_property_api = string_property.get_value_as_string();
assert(string_from_property_api == string);
value_got_from_property = string_property.get_value();
assert(value_got_from_property.as_ical_string() == string);
string_property.set_value_from_string(string, ICalGLib.Value.kind_to_string(kind));
string_from_property_api = string_property.get_value_as_string();
assert(string_from_property_api == string);
value_got_from_property = string_property.get_value();
assert(value_got_from_property.as_ical_string() == string);

string_property.set_x_name("This is an x name!");
assert(string_property.get_x_name() == "This is an x name!");

valuekind_from_parametervalue = ICalGLib.Parameter.value_to_value_kind(ICalGLib.ParameterValue.BINARY);
assert(valuekind_from_parametervalue == ICalGLib.ValueKind.BINARY_VALUE);

valuekind_from_property_kind = ICalGLib.Property.kind_to_value_kind(ICalGLib.PropertyKind.ACKNOWLEDGED_PROPERTY);
assert(valuekind_from_property_kind == ICalGLib.ValueKind.DATETIME_VALUE);
propertyKind = ICalGLib.Value.kind_to_property_kind(valuekind_from_property_kind);
assert(propertyKind == ICalGLib.PropertyKind.ACKNOWLEDGED_PROPERTY);
string = ICalGLib.Property.kind_to_string(ICalGLib.PropertyKind.ACKNOWLEDGED_PROPERTY);
assert(string == "ACKNOWLEDGED");
kind = ICalGLib.Property.kind_from_string(string);
assert(kind == ICalGLib.PropertyKind.ACKNOWLEDGED_PROPERTY);

string = ICalGLib.Property.method_to_string(ICalGLib.PropertyMethod.PUBLISH);
assert(ICalGLib.Property.method_from_string(string) == ICalGLib.PropertyMethod.PUBLISH);

string = ICalGLib.Property.enum_to_string(ICalGLib.PropertyMethod.PUBLISH);
assert(string == "PUBLISH");

#Test i_cal_value_set_parent. No error will be thrown.
property_string = "SUMMARY:Bastille Day Party";
string_property = ICalGLib.Property.new_from_string(property_string);
kind = ICalGLib.ValueKind.ATTACH_VALUE
string = "This is a link"
value_from_string = ICalGLib.Value.new_from_string(kind, string)
value_from_string.set_parent(string_property)
value_from_string.set_parent(None)



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./array.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

array = ICalGLib.Timezone.array_new();

#TEST APPEND
zone0 = ICalGLib.Timezone.get_builtin_timezone("Pacific/Midway");
zone1 = ICalGLib.Timezone.get_builtin_timezone("America/Vancouver");
zone2 = ICalGLib.Timezone.get_builtin_timezone("Atlantic/Bermuda");
zone3 = ICalGLib.Timezone.get_builtin_timezone("Africa/Casablanca");
zone4 = ICalGLib.Timezone.get_builtin_timezone("Asia/Irkutsk");

ICalGLib.Timezone.array_append_from_vtimezone(array, zone0.get_component());
ICalGLib.Timezone.array_append_from_vtimezone(array, zone1.get_component());
ICalGLib.Timezone.array_append_from_vtimezone(array, zone2.get_component());
ICalGLib.Timezone.array_append_from_vtimezone(array, zone3.get_component());
ICalGLib.Timezone.array_append_from_vtimezone(array, zone4.get_component());

assert array.size() == 5

z0 = ICalGLib.Timezone.array_element_at(array, 0);
assert(z0.get_location() == zone0.get_location());
z1 = ICalGLib.Timezone.array_element_at(array, 1);
assert(z1.get_location() == zone1.get_location());
z2 = ICalGLib.Timezone.array_element_at(array, 2);
assert(z2.get_location() == zone2.get_location());
z3 = ICalGLib.Timezone.array_element_at(array, 3);
assert(z3.get_location() == zone3.get_location());
z4 = ICalGLib.Timezone.array_element_at(array, 4);
assert(z4.get_location() == zone4.get_location());

array2 = array.copy();

assert array2.size() == 5

z0 = ICalGLib.Timezone.array_element_at(array2, 0);
assert(z0.get_location() == zone0.get_location());
z1 = ICalGLib.Timezone.array_element_at(array2, 1);
assert(z1.get_location() == zone1.get_location());
z2 = ICalGLib.Timezone.array_element_at(array2, 2);
assert(z2.get_location() == zone2.get_location());
z3 = ICalGLib.Timezone.array_element_at(array2, 3);
assert(z3.get_location() == zone3.get_location());
z4 = ICalGLib.Timezone.array_element_at(array2, 4);
assert(z4.get_location() == zone4.get_location());

array.remove_element_at(2);
assert array.size() == 4

z0 = ICalGLib.Timezone.array_element_at(array, 0);
assert(z0.get_location() == zone0.get_location());
z1 = ICalGLib.Timezone.array_element_at(array, 1);
assert(z1.get_location() == zone1.get_location());
z3 = ICalGLib.Timezone.array_element_at(array, 2);
assert(z3.get_location() == zone3.get_location());
z4 = ICalGLib.Timezone.array_element_at(array, 3);
assert(z4.get_location() == zone4.get_location());



#!/usr/bin/env python
#======================================================================
# FILE: Time.py
# CREATOR: eric
#
# (C) COPYRIGHT 2001, Eric Busboom <eric@civicknowledge.com>
# (C) COPYRIGHT 2001, Patrick Lewis <plewis@inetarena.com>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#======================================================================

from LibicalWrap import *
from Property import Property
from types import DictType, StringType, IntType, FloatType
from Duration import Duration

UTC = icaltimezone_get_utc_timezone()

class Time(Property):
    """ Represent iCalendar DATE, TIME and DATE-TIME """
    def __init__(self, arg, name="DTSTART", zone=None):
        """
        Create a new Time from a string or number of seconds past the
        POSIX epoch

        Time("19970325T123000Z")  Construct from an iCalendar string
        Time(8349873494)          Construct from seconds past POSIX epoch

        """
        e1=icalerror_supress("MALFORMEDDATA")
        e2=icalerror_supress("BADARG")

        if isinstance(arg, DictType):
            # Dictionary -- used for creating from Component
            self.tt = icaltime_from_string(arg['value'])
            Property.__init__(self, ref=arg['ref'])
        else:
            if isinstance(arg, StringType):
                # Create from an iCal string
                self.tt = icaltime_from_string(arg)
            elif isinstance(arg, IntType) or   \
                 isinstance(arg, FloatType):
                # Create from seconds past the POSIX epoch
                if zone:
                        self.tt = icaltime_from_timet_with_zone(int(arg),0,icaltimezone_get_builtin_timezone(zone))
                else:
                        self.tt = icaltime_from_timet_with_zone(int(arg),0,icaltimezone_get_utc_timezone())
            elif isinstance(arg, Time):
                # Copy an instance
                self.tt = arg.tt
            else:
                self.tt = icaltime_null_time()

            Property.__init__(self,type=name)

        icalerror_restore("MALFORMEDDATA",e1)
        icalerror_restore("BADARG",e2)

        if icaltime_is_null_time(self.tt):
            raise Property.ConstructorFailedError("Failed to construct a Time")

        try:
            self._update_value()
        except Property.UpdateFailedError:
            raise Property.ConstructorFailedError("Failed to construct a Time")

    def _update_value(self):
        self.normalize()
        self.value(icaltime_as_ical_string(self.tt),"DATE-TIME")

    def valid(self):
        " Returns true if this is a valid time "
        return not icaltime_is_null_time(self.tt)

    def utc_seconds(self,v=None):
        """ Returns or sets time in seconds past POSIX epoch"""
        tz = icaltimezone_get_builtin_timezone(self.timezone())
        if (v!=None):
            self.tt = icaltime_from_timet_with_zone(v,0,tz)
            self._update_value()

        return icaltime_as_timet_with_zone(self.tt, tz)

    def is_utc(self):
        """ Returns a boolean indicating if time is in UTC """
        return icaltime_is_utc(self.tt)

    def is_date(self):
        """ Returns a boolean indicating if time is actually a date """
        return icaltime_is_date(self.tt)

    def timezone(self,v=None):
        """ Returns, set (if none) or alter the timezone for this time """

        origtz = icaltime_get_tzid(self.tt)

        if (v != None):
            assert(isinstance(v,StringType) )
            if (v == "UTC"):
                tz = icaltimezone_get_utc_timezone()
                del self['TZID']
            else:
                tz = icaltimezone_get_builtin_timezone(v)

            if not origtz:
                self.tt = icaltime_set_timezone(self.tt, tz)
            else:
                self.tt = icaltime_convert_to_zone(self.tt,tz)

            if (icaltime_get_tzid(self.tt) != "UTC"):
                self['TZID'] = icaltime_get_tzid(self.tt)

        self._update_value()
        return icaltime_get_tzid(self.tt)

    def normalize(self):
        self.tt = icaltime_normalize(self.tt)

    def __second_property(self,v=None):
        """ Get or set the seconds component of this time """
        if(v != None):
            self.tt.second = v
            self._update_value()
        return self.tt.second
    second = property(__second_property, __second_property)

    def __minute_property(self,v=None):
        """ Get or set the minute component of this time """
        if(v != None):
            self.tt.minute = v
            self._update_value()
        return self.tt.minute
    minute = property(__minute_property, __minute_property)

    def __hour_property(self,v=None):
        """ Get or set the hour component of this time """
        if(v != None):
            self.tt.hour = v
            self._update_value()
        return self.tt.hour
    hour = property(__hour_property, __hour_property)

    def __day_property(self,v=None):
        """ Get or set the month day component of this time """
        if(v != None):
            self.tt.day = v
            self._update_value()
        return self.tt.day
    day = property(__day_property, __day_property)

    def __month_property(self,v=None):
        """ Get or set the month component of this time. January is month 1 """
        if(v != None):
            self.tt.month = v
            self._update_value()
        return self.tt.month
    month = property(__month_property, __month_property)

    def __year_property(self,v=None):
        """ Get or set the year component of this time """
        if(v != None):
            self.tt.year = v
            self._update_value()
        return self.tt.year
    year = property(__year_property, __year_property)


    def __cmp__(self,other):

        if other == None:
            return cmp(self.utc_seconds(),None)

        return cmp(self.utc_seconds(),other.utc_seconds())


    def __add__(self,o):

        other = Duration(o,"DURATION")

        if not other.valid():
            return Duration(0,"DURATION")

        print self.utc_seconds(), other.seconds()
        seconds = self.utc_seconds() + other.seconds()

        new = Time(seconds,self.name(),self.timezone())

        return new

    def __radd_(self,o):
        return self.__add__(o)


    def __sub__(self,o):


        if isinstance(o,Time):
            # Subtract a time from this time and return a duration
            seconds = self.utc_seconds() - o.utc_seconds()
            return Duration(seconds)
        elif isinstance(o,Duration):
            # Subtract a duration from this time and return a time
            other = Duration(o)
            if(not other.valid()):
                return Time()

            seconds = self.utc_seconds() - other.seconds()
            return Time(seconds)
        else:
            raise TypeError, "subtraction with Time reqires Time or Duration"



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./recurrence-type.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

weekday = ICalGLib.Recurrence.day_day_of_week(0)
assert(weekday == ICalGLib.RecurrenceWeekday.NO_WEEKDAY)
weekday = ICalGLib.Recurrence.day_day_of_week(1)
assert(weekday == ICalGLib.RecurrenceWeekday.SUNDAY_WEEKDAY)

assert(ICalGLib.Recurrence.day_position(15) == 1)
assert(ICalGLib.Recurrence.day_position(16) == 2)
assert(ICalGLib.Recurrence.day_position(25) == 3)

string = "COUNT=10;FREQ=DAILY"
recurrence = ICalGLib.Recurrence.new_from_string(string)
assert(recurrence.to_string() == "FREQ=DAILY;COUNT=10")

by_second = recurrence.get_by_second_array()
# The value is dependent on the libical version.
assert len(by_second) == 61 or len(by_second) == 62
by_minute = recurrence.get_by_minute_array()
assert len(by_minute) == 61
by_hour = recurrence.get_by_hour_array()
assert len(by_hour) == 25
by_day = recurrence.get_by_day_array()
# The value is dependent on the libical version.
assert len(by_day) == 364 or len(by_day) == 386
by_month_day = recurrence.get_by_month_day_array()
assert len(by_month_day) == 32
by_year_day = recurrence.get_by_year_day_array()
# The value is dependent on the libical version.
assert len(by_year_day) == 367 or len(by_year_day) == 386
by_week_no = recurrence.get_by_week_no_array()
# The value is dependent on the libical version.
assert len(by_week_no) == 54 or len(by_week_no) == 56
by_month = recurrence.get_by_month_array()
# The value is dependent on the libical version.
assert len(by_month) == 13 or len(by_month) == 14
by_set_pos = recurrence.get_by_set_pos_array()
# The value is dependent on the libical version.
assert len(by_set_pos) == 367 or len(by_set_pos) == 386

recurrence.set_by_second(0, 1)
recurrence.set_by_second(1, ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX)
assert recurrence.get_by_second(0) == 1
assert recurrence.get_by_second(1) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_second_array()
assert array[0] == 1
assert array[1] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = [ 100, 101 ]
assert len(array) == 2
recurrence.set_by_second_array(array)
assert recurrence.get_by_second(0) == 100
assert recurrence.get_by_second(1) == 101
assert recurrence.get_by_second(2) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_second_array()
assert array[0] == 100
assert array[1] == 101
assert array[2] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX

recurrence.set_by_minute(0, 2)
recurrence.set_by_minute(1, ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX)
assert recurrence.get_by_minute(0) == 2
assert recurrence.get_by_minute(1) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_minute_array()
assert array[0] == 2
assert array[1] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = [ 200, 201 ]
assert len(array) == 2
recurrence.set_by_minute_array(array)
assert recurrence.get_by_minute(0) == 200
assert recurrence.get_by_minute(1) == 201
assert recurrence.get_by_minute(2) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_minute_array()
assert array[0] == 200
assert array[1] == 201
assert array[2] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX

recurrence.set_by_hour(0, 3)
recurrence.set_by_hour(1, ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX)
assert recurrence.get_by_hour(0) == 3
assert recurrence.get_by_hour(1) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_hour_array()
assert array[0] == 3
assert array[1] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = [ 300, 301 ]
assert len(array) == 2
recurrence.set_by_hour_array(array)
assert recurrence.get_by_hour(0) == 300
assert recurrence.get_by_hour(1) == 301
assert recurrence.get_by_hour(2) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_hour_array()
assert array[0] == 300
assert array[1] == 301
assert array[2] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX

recurrence.set_by_day(0, 4)
recurrence.set_by_day(1, ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX)
assert recurrence.get_by_day(0) == 4
assert recurrence.get_by_day(1) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_day_array()
assert array[0] == 4
assert array[1] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = [ 400, 401 ]
assert len(array) == 2
recurrence.set_by_day_array(array)
assert recurrence.get_by_day(0) == 400
assert recurrence.get_by_day(1) == 401
assert recurrence.get_by_day(2) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_day_array()
assert array[0] == 400
assert array[1] == 401
assert array[2] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX

recurrence.set_by_month_day(0, 5)
recurrence.set_by_month_day(1, ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX)
assert recurrence.get_by_month_day(0) == 5
assert recurrence.get_by_month_day(1) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_month_day_array()
assert array[0] == 5
assert array[1] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = [ 500, 501 ]
assert len(array) == 2
recurrence.set_by_month_day_array(array)
assert recurrence.get_by_month_day(0) == 500
assert recurrence.get_by_month_day(1) == 501
assert recurrence.get_by_month_day(2) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_month_day_array()
assert array[0] == 500
assert array[1] == 501
assert array[2] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX

recurrence.set_by_year_day(0, 6)
recurrence.set_by_year_day(1, ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX)
assert recurrence.get_by_year_day(0) == 6
assert recurrence.get_by_year_day(1) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_year_day_array()
assert array[0] == 6
assert array[1] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = [ 600, 601 ]
assert len(array) == 2
recurrence.set_by_year_day_array(array)
assert recurrence.get_by_year_day(0) == 600
assert recurrence.get_by_year_day(1) == 601
assert recurrence.get_by_year_day(2) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_year_day_array()
assert array[0] == 600
assert array[1] == 601
assert array[2] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX

recurrence.set_by_week_no(0, 7)
recurrence.set_by_week_no(1, ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX)
assert recurrence.get_by_week_no(0) == 7
assert recurrence.get_by_week_no(1) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_week_no_array()
assert array[0] == 7
assert array[1] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = [ 700, 701 ]
assert len(array) == 2
recurrence.set_by_week_no_array(array)
assert recurrence.get_by_week_no(0) == 700
assert recurrence.get_by_week_no(1) == 701
assert recurrence.get_by_week_no(2) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_week_no_array()
assert array[0] == 700
assert array[1] == 701
assert array[2] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX

recurrence.set_by_month(0, 8)
recurrence.set_by_month(1, ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX)
assert recurrence.get_by_month(0) == 8
assert recurrence.get_by_month(1) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_month_array()
assert array[0] == 8
assert array[1] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = [ 800, 801 ]
assert len(array) == 2
recurrence.set_by_month_array(array)
assert recurrence.get_by_month(0) == 800
assert recurrence.get_by_month(1) == 801
assert recurrence.get_by_month(2) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_month_array()
assert array[0] == 800
assert array[1] == 801
assert array[2] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX

recurrence.set_by_set_pos(0, 9)
recurrence.set_by_set_pos(1, ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX)
assert recurrence.get_by_set_pos(0) == 9
assert recurrence.get_by_set_pos(1) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_set_pos_array()
assert array[0] == 9
assert array[1] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = [ 900, 901 ]
assert len(array) == 2
recurrence.set_by_set_pos_array(array)
assert recurrence.get_by_set_pos(0) == 900
assert recurrence.get_by_set_pos(1) == 901
assert recurrence.get_by_set_pos(2) == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX
array = recurrence.get_by_set_pos_array()
assert array[0] == 900
assert array[1] == 901
assert array[2] == ICalGLib.RecurrenceArrayMaxValues.RECURRENCE_ARRAY_MAX

recurrence.set_by_second(0, 13)
by_second = recurrence.get_by_second_array()
assert by_second[0] == 13

recurrence = ICalGLib.Recurrence.new_from_string(string)

assert(ICalGLib.Recurrence.weekday_from_string("MO") == ICalGLib.RecurrenceWeekday.MONDAY_WEEKDAY)

start = 100000
result = ICalGLib.recur_expand_recurrence(string, start, 10)
secs_per_day = 24*60*60
for i in range(0, 9):
    assert(result[i] == start + i*secs_per_day)

string = "19970101T183248Z/19970102T071625Z"

period = ICalGLib.Period.new_from_string(string)
start = period.get_start()

iter = ICalGLib.RecurIterator.new(recurrence, start)
timetype = iter.next()
day = timetype.get_day()
ref = 1
while day != 0:
    assert(day == ref)
    ref += 1
    timetype = iter.next()
    day = timetype.get_day()

recurrence = ICalGLib.Recurrence.new_from_string("FREQ=YEARLY;COUNT=10")
start = ICalGLib.Time.new_from_string("20161224T000000Z")

iter = ICalGLib.RecurIterator.new(recurrence, start)

start = ICalGLib.Time.new_from_string("20181224T000000Z")

assert ICalGLib.RecurIterator.set_start(iter, start) == 0

recurrence = ICalGLib.Recurrence.new_from_string("FREQ=YEARLY")
start = ICalGLib.Time.new_from_string("20161224T000000Z")

iter = ICalGLib.RecurIterator.new(recurrence, start)

start = ICalGLib.Time.new_from_string("20181224T000000Z")

assert ICalGLib.RecurIterator.set_start(iter, start) == 1
timetype = iter.next()

assert timetype.get_year() == 2018

timetype = iter.next()
assert timetype.get_year() == 2019
